@@ -32,6 +32,8 @@ import traceback
 import urllib
 import urlparse
 
+from thrift.Thrift import TType
+
 import flawless.lib.config
 from flawless.lib.data_structures import prefix_tree
 from flawless.lib.storage import DiskStorage
@@ -59,8 +61,8 @@ def code_identifier_equality(self, other):
         return False
 
     # Compress whitespace characters to make the comparissions more forgiving
-    self_fragment = None if not self.code_fragment else re.sub("\s+", " ", self.code_fragment)
-    other_fragment = None if not other.code_fragment else re.sub("\s+", " ", other.code_fragment)
+    self_fragment = None if not self.code_fragment else re.sub("\s+", " ", self.code_fragment.strip())
+    other_fragment = None if not other.code_fragment else re.sub("\s+", " ", other.code_fragment.strip())
     if (self_fragment and other_fragment and self_fragment not in other_fragment and
             other_fragment not in self_fragment):
         return False
@@ -78,13 +80,14 @@ class FlawlessServiceBaseClass(object):
     """Contains numerous shared helper functions for the ThriftService & WebService classes"""
 
     def __init__(self,
-                 storage_cls=DiskStorage,
+                 storage_factory=DiskStorage,
                  thread_cls=threading.Thread,
                  open_process_func=subprocess.Popen,
                  smtp_client_cls=smtplib.SMTP,
                  time_func=time.time):
         self.__dict__.update({k: v for k, v in locals().iteritems() if k != 'self'})
-        self.extract_base_path_pattern = re.compile('^.*/%s/?(.*)$' % config.report_runtime_package_directory_name)
+        self.extract_base_path_patterns = [re.compile('^.*/%s/?(.*)$' % d) for d in
+                                           config.report_runtime_package_directory_names]
         self.raise_exception_pattern = re.compile('^\s*raise[ \n]')
         self.only_blame_patterns = [re.compile(p) for p in config.only_blame_filepaths_matching]
         self._read_whitelist_configs()
@@ -92,7 +95,7 @@ class FlawlessServiceBaseClass(object):
     ############################## Parse Config Files ##############################
 
     def _read_whitelist_configs(self):
-        config_storage = self.storage_cls(partition=None)
+        config_storage = self.storage_factory(partition=None)
         config_storage.open()
 
         # Whitelists
@@ -156,11 +159,10 @@ class FlawlessServiceBaseClass(object):
     ############################## Traceback/Line Type Helpers ##############################
 
     def _get_line_type(self, line):
-        match = self.extract_base_path_pattern.match(line.filename)
-        if not match:
+        filepath = self._get_basepath(line.filename)
+        if not filepath:
             return api_ttypes.LineType.BAD_FILEPATH
 
-        filepath = match.group(1)
         entry = api_ttypes.CodeIdentifier(filepath, line.function_name, line.text)
         if entry in self.third_party_whitelist[filepath]:
             return api_ttypes.LineType.THIRDPARTY_WHITELIST
@@ -186,6 +188,13 @@ class FlawlessServiceBaseClass(object):
                 return True
         return False
 
+    def _get_basepath(self, filename):
+        for pattern in self.extract_base_path_patterns:
+            match = pattern.match(filename)
+            if match:
+                return match.group(1)
+        return None
+
     def _format_traceback(self, request, append_locals=True, show_full_stack=False,
                           linebreak="<br />", spacer="&nbsp;", start_bold="<strong>",
                           end_bold="</strong>", escape_func=cgi.escape):
@@ -267,12 +276,12 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
         prefix = self._partition_for_ms(epoch_ms)
         with self.lock:
             if self.errors_seen is None:
-                self.errors_seen = self.storage_cls(prefix)
+                self.errors_seen = self.storage_factory(prefix)
                 self.errors_seen.open()
             elif prefix != self.errors_seen.partition:
                 # Order matters here since there can be a race condition if not done correctly
                 old_errors_seen = self.errors_seen
-                new_errors_seen = self.storage_cls(prefix)
+                new_errors_seen = self.storage_factory(prefix)
                 new_errors_seen.open()
                 self.errors_seen = new_errors_seen
                 old_errors_seen.sync()
@@ -313,7 +322,7 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
             )
 
         msg = email.MIMEText.MIMEText(body.encode("UTF-8"), "html", "UTF-8")
-        msg["From"] = "error_report@%s" % config.email_domain_name
+        msg["From"] = "flawless@%s" % config.email_domain_name
         msg["To"] = ", ".join(to_addresses)
         msg["Subject"] = subject
 
@@ -366,7 +375,7 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
             if line_type == api_ttypes.LineType.THIRDPARTY_WHITELIST:
                 return None, None, None, True
             elif line_type in [api_ttypes.LineType.DEFAULT, api_ttypes.LineType.KNOWN_ERROR]:
-                filepath = self.extract_base_path_pattern.match(stack_line.filename).group(1)
+                filepath = self._get_basepath(stack_line.filename)
                 entry = api_ttypes.CodeIdentifier(filepath, stack_line.function_name, stack_line.text)
                 blamed_entry = entry
                 key = api_ttypes.ErrorKey(filepath, stack_line.line_number, stack_line.function_name, stack_line.text)
@@ -375,7 +384,7 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
         return (key, blamed_entry, email_recipients, False)
 
     def _record_error(self, request):
-        log.info("Recieved error from %s for %s" % (request.hostname, request.exception_message))
+        log.debug("Recieved error from %s for %s" % (request.hostname, request.exception_message))
 
         # Figure out which line in the stack trace is to blame for the error
         key, blamed_entry, email_recipients, was_whitelisted = self._blame_line(request.traceback)
@@ -478,7 +487,7 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
             email_body.append(
                 "<br /><br /><a href='http://%s/add_known_error?%s'>Add to whitelist</a>" %
                 (
-                    config.hostname + ":" + str(config.port),
+                    config.hostname,
                     urllib.urlencode(
                         dict(filename=key.filename, function_name=key.function_name, code_fragment=key.text)
                     )
@@ -509,7 +518,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
 
     def _get_errors_seen_for_ts(self, timestamp):
         prefix = self._partition_for_ms(int(timestamp) * 1000 if timestamp else None)
-        errors_seen = self.storage_cls(prefix)
+        errors_seen = self.storage_factory(prefix)
         errors_seen.open()
         return errors_seen
 
@@ -518,7 +527,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
                          building_blocks=api_ttypes.CodeIdentifierList,
                          third_party_whitelist=api_ttypes.CodeIdentifierList,
                          watch_list=api_ttypes.WatchList)
-        config_storage = self.storage_cls(partition=None)
+        config_storage = self.storage_factory(partition=None)
         config_storage.open()
         current_value = config_storage[key] or class_map[key]()
         getattr(current_value, attr).append(entry)
@@ -528,9 +537,28 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
         config_storage.close()
 
     def _construct_instance(self, params, cls):
+        THRIFT_SPEC_NAME_FIELD = 2
+        THRIFT_SPEC_TYPE_FIELD = 1
         init_args = inspect.getargspec(cls.__init__).args
         whitelist_attrs = [s for s in init_args if s != 'self']
-        new_entry = cls(**dict((k, params.get(k)) for k in whitelist_attrs))
+        args = dict((k, params.get(k)) for k in whitelist_attrs)
+        arg_type_map = dict()
+
+        # Build map of fields to type based on the thrift spec
+        for spec in cls.thrift_spec:
+            if not spec or len(spec) < 3:
+                continue
+            if spec[THRIFT_SPEC_TYPE_FIELD] == TType.BOOL:
+                arg_type_map[spec[THRIFT_SPEC_NAME_FIELD]] = bool
+            elif spec[THRIFT_SPEC_TYPE_FIELD] == TType.I64 or spec[THRIFT_SPEC_TYPE_FIELD] == TType.I32:
+                arg_type_map[spec[THRIFT_SPEC_NAME_FIELD]] = int
+
+        # Cast args to the appropriate type
+        for field, value in args.items():
+            if value and field in arg_type_map:
+                args[field] = arg_type_map[field](value)
+
+        new_entry = cls(**args)
         return new_entry
 
     def get_weekly_error_report(self, timestamp=None, include_known_errors=False,
@@ -559,8 +587,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
                 params = copy.copy(err_key.__dict__)
                 if timestamp:
                     params["timestamp"] = timestamp
-                view_url = "http://%s/view_traceback?%s" % (config.hostname + ":" + str(config.port),
-                                                            urllib.urlencode(params))
+                view_url = "%s/view_traceback?%s" % (config.hostname, urllib.urlencode(params))
                 html_parts.append("<a href='%s'>view traceback</a>" % view_url)
                 html_parts.append("<br />")
             html_parts.append("<br />")
@@ -572,7 +599,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
     def view_traceback(self, filename="", function_name="", text="", line_number="", timestamp=None):
         errors_seen = self._get_errors_seen_for_ts(timestamp)
         err_key = api_ttypes.ErrorKey(filename=filename, function_name=function_name,
-                                      text=text, line_number=line_number)
+                                      text=text, line_number=int(line_number))
 
         err_info = errors_seen[err_key]
         if err_info:
@@ -746,7 +773,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
 
     def save_remap_email(self, request):
         params = dict(urlparse.parse_qsl(request))
-        config_storage = self.storage_cls(partition=None)
+        config_storage = self.storage_factory(partition=None)
         config_storage.open()
         current_value = config_storage["email_remapping"] or api_ttypes.EmailRemapping()
         current_value.remap[params["old_email"]] = params["new_email"]
@@ -757,7 +784,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
         return "<html><body>SUCCESS</body></html>"
 
     def view_config(self, key):
-        config_storage = self.storage_cls(partition=None)
+        config_storage = self.storage_factory(partition=None)
         config_storage.open()
         current_value = config_storage[key]
         config_storage.close()
